function [ boolean ] = arrival_of_profiles_overview( p_nemo_data, p_config )
%ARRIVAL_OF_PROFILES Plots details about iridium data.
%
%   Returns:
%   boolean: False if something went wrong during the plotting process.
%
%   Parameters:
%   p_nemo_data:      Struct, containing all float data (load with nemo.load.data()).
%   p_config:         Global configuration file. Load using nemo.load.config().

%% Initialize return variable
boolean = false;

%% Parameter check
parameter_error = false;

if (~iscell(p_nemo_data) || ~isstruct(p_config))
    parameter_error = true;
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Setup directories
directory = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.storage_directory_image);
directory_fig = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.storage_directory_fig);
image_type = p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.image_type;
if ~exist(directory, 'dir')
    system(['mkdir ' directory]);
end
if ~exist(directory_fig, 'dir')
    mkdir(directory_fig);
end


%% Collect required data

first_deployment_datenum = NaN; % contains the earliest deployment date of all floats
latest_profile_datenum = NaN; % contains the latest profile that has been transmitted

best_guess_datenum = {};
transmitted_profiles = {};
iridium_mean_times = {};
float_serials = [];

float_count = length(p_nemo_data);

for o_float = 1:float_count

    % get serial number
    float_serials = [float_serials; p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number];
    
    % get the deployment dates
    current_deployment_datenum = datenum( ...
        [ ...
          p_nemo_data{o_float}{1}.DEPLOYMENT_INFO.deployment_date ...
          p_nemo_data{o_float}{1}.DEPLOYMENT_INFO.deployment_time ...
          0 ...
        ] ...
    );
    if isnan(first_deployment_datenum) || (current_deployment_datenum < first_deployment_datenum)
        first_deployment_datenum = current_deployment_datenum;
    end
    current_latest_profile_datenum = datenum(p_nemo_data{o_float}{end}.SECOND_ORDER_INFORMATION.best_guess_datevec);
    % get latest profile date
    if ~all(isnan(current_latest_profile_datenum)) && (isnan(latest_profile_datenum) || (current_latest_profile_datenum > latest_profile_datenum))
        latest_profile_datenum = current_latest_profile_datenum;
    end
    
    % check if gps data available
    gps_datetime = nemo.float.collect(p_nemo_data{o_float}, ...
        'SURFACE_GPS_DATA', ...
        '' ...
    );
    if isempty(gps_datetime)
        continue;
    end
    
    
    % collect required data
    
    best_guess_datevec = nemo.float.collect(p_nemo_data{o_float}, ...
        'SECOND_ORDER_INFORMATION', ...
        'best_guess_datevec' ...
        );
    best_guess_datenum{o_float} = datenum(best_guess_datevec);
    
    % get numbers of the profiles that have been received
    transmitted_profiles{o_float} = nemo.float.collect(p_nemo_data{o_float}, ...
        'PROFILE_TECHNICAL_DATA', ...
        'xmit_profile_number' ...
    );

    % average time when the profile has been transmitted
    iridium_mean_times{o_float} = nemo.float.collect( ...
        p_nemo_data{o_float}, ...
        'IRIDIUM_DATA', ...
        '' ...
    );
    % extract only datevec from it
    iridium_mean_times{o_float} = datetime(iridium_mean_times{o_float}(:, 4:9));
    
    
end

% get min and max datevec
if (first_deployment_datenum == latest_profile_datenum)
    latest_profile_datenum = latest_profile_datenum + 1;
end
diff_datenum = latest_profile_datenum - first_deployment_datenum;
if (diff_datenum > 365)
    x_tick = [first_deployment_datenum:365:latest_profile_datenum];
elseif (diff_datenum > 30)
    x_tick = [first_deployment_datenum:30:latest_profile_datenum];
elseif (diff_datenum > 5)
    x_tick = [first_deployment_datenum:5:latest_profile_datenum];
else
    x_tick = [first_deployment_datenum:1:latest_profile_datenum];
end
x_tick = [ x_tick(1:end-1) latest_profile_datenum ];
    
% get plot details
width = p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.image_width;
height = p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.image_height;
paperunit = p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.image_paperunit;

% plot collected data

figure_id = figure('Visible', 'off'); % create figure
% initialize figure
set(figure_id,'InvertHardcopy','on');
set(figure_id,'PaperUnits', paperunit);
papersize = get(figure_id, 'PaperSize');
left = (papersize(1)- width)/2;
bottom = (papersize(2)- height)/2;
myfiguresize = [left, bottom, width, height];
set(figure_id,'PaperPosition', myfiguresize);
clear height width left bottom papersize myfiguresize

%set(gcf, 'Position', get(0,'ScreenSize'), 'color', [1 1 1]);
sub = {};

sub{1} = subplot(2,1,1);
hold on
%     set(gca, 'XLimMode', 'manual', 'XLim', [min_datenum, max_datenum]);
%     set_x_axis();
for o_float = 1:float_count
    y = repmat(float_serials(o_float), length(transmitted_profiles{o_float}), 1);
    plot(transmitted_profiles{o_float}, y,'marker','.','color',[0 0 0],'linestyle','none','linewidth',1,'markersize',20);
    clear y
end

t = title([ 'Arrival of profiles overview @ ' ...
      datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
    ]);
set(t, 'FontSize', 18);
clear t
ylabel('Float serial number');
xlabel('Profile number');
grid on
box on
hold off



sub{2} = subplot(2,1,2);
hold on
%     set(gca, 'XLimMode', 'manual', 'XLim', [min_datenum, max_datenum]);
%     set_x_axis();
for o_float = 1:float_count
    
    disp(['   Processing arrival overview information of float number ',num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)])  % OB
    
    y = repmat(float_serials(o_float), length(best_guess_datenum{o_float}), 1);
    plot(best_guess_datenum{o_float}, y,'marker','.','color',[0.5 0.5 0.5],'linestyle','none','linewidth',1,'markersize',20);
    y = repmat(float_serials(o_float), length(transmitted_profiles{o_float}), 1);
            
    % OB START: for float 173 iridium_mean_times is of type datetime, which does not
    % comply with plot command
    % Old Code:
    % plot(iridium_mean_times{o_float}, y,'marker','.','color',[0 0 0],'linestyle','none','linewidth',1,'markersize',20);
    % New Code
    if isa(iridium_mean_times{o_float},'double')
        plot(iridium_mean_times{o_float}, y,'marker','.','color',[0 0 0],'linestyle','none','linewidth',1,'markersize',20);
    elseif isa(iridium_mean_times{o_float},'datetime')
        plot(datenum(iridium_mean_times{o_float}), y,'marker','.','color',[0 0 0],'linestyle','none','linewidth',1,'markersize',20);
    end
    % OB END
    
    clear y
end

ylabel('Float serial number');
xlabel('Iridium timestamp');
grid on
box on
hold off

% set_x_axis(sub{2});


% store plot

image_filename = p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.image_filename;
figure_filename = p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.figure_filename;

print(figure_id, [directory image_filename], image_type);

 if (p_config.PLOT_ARRIVAL_OF_PROFILES_OVERVIEW.save_figure)
     saveas(figure_id, ...
         [directory_fig figure_filename]);
 end

close(figure_id);

boolean = true;


    function current_plot = set_x_axis(current_plot)
        set(current_plot, 'XLimMode', 'manual', ...
                          'XLim', [first_deployment_datenum, latest_profile_datenum], ...
                          'XTick', x_tick);
        datetick(current_plot, 'x', 'yyyy-mm-dd', 'keepticks');
        
    end

end

